import json
import time
from warnings import warn

import requests
from packaging.version import Version
from snowflake.cli.__about__ import VERSION
from snowflake.cli.api.cli_global_context import get_cli_context
from snowflake.cli.api.secure_path import SecurePath
from snowflake.connector.config_manager import CONFIG_MANAGER

REPOSITORY_URL_PIP = "https://pypi.org/pypi/snowflake-cli/json"
REPOSITORY_URL_BREW = "https://formulae.brew.sh/api/formula/snowflake-cli.json"


def get_new_version_msg() -> str | None:
    last = _VersionCache().get_last_version()
    current = Version(VERSION)
    if last and last > current:
        return f"\nNew version of Snowflake CLI available. Newest: {last}, current: {VERSION}\n"
    return None


def show_new_version_banner_callback(msg):
    def _callback(*args, **kwargs):
        if msg and not get_cli_context().silent:
            warn(msg)

    return _callback


class _VersionCache:
    _last_time = "last_time_check"
    _version = "version"
    _version_cache_file = SecurePath(
        CONFIG_MANAGER.file_path.parent / ".cli_version.cache"
    )

    def __init__(self):
        self._cache_file = _VersionCache._version_cache_file

    def _save_latest_version(self, version: str):
        data = {
            _VersionCache._last_time: time.time(),
            _VersionCache._version: str(version),
        }
        self._cache_file.write_text(json.dumps(data))

    @staticmethod
    def _get_version_from_pypi() -> str | None:
        headers = {"Content-Type": "application/vnd.pypi.simple.v1+json"}
        response = requests.get(REPOSITORY_URL_PIP, headers=headers, timeout=3)
        response.raise_for_status()
        return response.json().get("info", {}).get("version", None)

    @staticmethod
    def _get_version_from_brew() -> str | None:
        response = requests.get(REPOSITORY_URL_BREW, timeout=3)
        response.raise_for_status()
        return response.json().get("versions", {}).get("stable", None)

    def _update_latest_version(self) -> Version | None:
        # Use brew version, fallback to pypi if brew is not available.
        # Brew repo takes longer to propagate the upgrade and is triggered later in our release process,
        # we treat it as "slowest point" and determinant that the released version is available.
        version = self._get_version_from_brew() or self._get_version_from_pypi()
        if version is None:
            return None
        self._save_latest_version(version)
        return Version(version)

    def _read_latest_version(self) -> Version | None:
        if self._cache_file.exists():
            data = json.loads(self._cache_file.read_text(file_size_limit_mb=1))
            now = time.time()
            if data[_VersionCache._last_time] > now - 60 * 60:
                return Version(data[_VersionCache._version])

        return self._update_latest_version()

    def get_last_version(self) -> Version | None:
        try:
            return self._read_latest_version()
        except:  # anything, this it not crucial feature
            return None
